#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "SDL_resource.h"
#include "ehash.h"
#include "bsicon-32x32.h"

#define SECOND_TICKS 1000L
#define MINUTE_TICKS (SECOND_TICKS*60L)
#define HOUR_TICKS   (MINUTE_TICKS*60L)
#define DAY_TICKS    (HOUR_TICKS*24L)

#if defined(WIN32)&&defined(_DEBUG)
#define RELPATH "../"
#else
#define RELPATH NULL
#endif

static const SDL_Color dgrey={ 98,105, 98,SDL_ALPHA_OPAQUE};
static const SDL_Color lgrey={156,153,156,SDL_ALPHA_OPAQUE};
static const SDL_Rect t_area={62,95,90,16};
static SDL_Surface *screen=NULL;

void checker_fill(SDL_Surface *s, int x, int y, int size,
  SDL_Color cs1, SDL_Color cs2);
void mod_time(Sint32 ticks, char *buf);

typedef enum eggres_e
{
  EGGRES_FONTGFX,
  EGGRES_BACKGND,
  EGGRES_ALARM,
  EGGRES_TICK,
  EGGRES_LAST,
  EGGRES_ERROR=-1
} eggres_e;

restype egg_resources[]={
	{RES_GRAPHIC,SRC_FILE,{"font_8x16.png"},{1,255,0,255},{NULL},1,0},
  {RES_GRAPHIC,SRC_FILE   ,{"stopwatch.png"},{0}  ,{NULL},1,0},
  {RES_SOUND  ,SRC_FILE   ,{"watchalm.wav"} ,{128},{NULL},1,0},
  {RES_SOUND  ,SRC_FILE   ,{"tick.wav"}     ,{128},{NULL},1,0},
  {RES_INVALID,SRC_INVALID,{NULL}           ,{0}  ,{NULL},0,0} };

void blank_screen()
{
  checker_fill(screen,0,0,8,dgrey,lgrey);
  SDL_BlitSurface(egg_resources[EGGRES_BACKGND].data.graphic,NULL,screen,NULL);
}

void stringRGBA(SDL_Surface *s, Sint16 x, Sint16 y, const char *str,
	Uint8 r, Uint8 g, Uint8 b)
{
  int n;
  int xoff=egg_resources[EGGRES_FONTGFX].data.graphic->w/16;
  int yoff=egg_resources[EGGRES_FONTGFX].data.graphic->h/16;
  SDL_Rect curpos={x,y,xoff,yoff};
  SDL_Color c={r,g,b,SDL_ALPHA_OPAQUE};

  SDL_SetColors(egg_resources[EGGRES_FONTGFX].data.graphic,
    &c,0,1);


  for(n=0; str[n]!='\0'; n++)
  {
    unsigned char c=str[n];
	SDL_Rect letterpos={(c%16)*xoff,(c/16)*yoff,xoff,yoff};
	SDL_BlitSurface(egg_resources[EGGRES_FONTGFX].data.graphic,
		&letterpos,screen,&curpos);
    curpos.x+=xoff;
  }
}

int main(int argc, char *argv[])
{
  char buf[32];
  Sint32 reference,timer=0;
  int running=1,counting=0,redraw=1;

  if(SDL_Init(SDL_INIT_VIDEO)<0)
  {
    fprintf(stderr,"Can't init SDL: %s\n",SDL_GetError());
    exit(1);
  }

  if(Mix_OpenAudio(22050,AUDIO_S16,1,1024)<0)
  {
    fprintf(stderr,"Can't init SDL_mixer: %s\n",SDL_GetError());
    exit(1);
  }

  Mix_AllocateChannels(1);

  if(SDL_LoadResources(egg_resources,RELPATH)<0)
  {
    fprintf(stderr,"Couldn't load resources: %s\n",SDL_GetError());
    exit(1);
  }

  SetIcon();
  screen=SDL_SetVideoMode(207,230,32,SDL_ANYFORMAT);
  if(screen==NULL)
  {
    fprintf(stderr,"Can't set video mode: %s\n",SDL_GetError());
    SDL_Quit();
    return(1);
  }

//  gfxPrimitivesSetFont(gfxPrimitivesFontdata,8,8);
  SDL_WM_SetCaption("Egg Timer","Egg Timer");

  reference=SDL_GetTicks();

  blank_screen();
  SDL_Flip(screen);

  while(running)
  {
    int toffset;
    Sint32 curtime=SDL_GetTicks();
    SDL_Event event;

    if(!counting)
      reference=curtime;

    while(SDL_PollEvent(&event))
    switch(EventHash(&event))
    {
    case SDL_MOUSEUP_H(SDL_BUTTON_LEFT,0):
      if(counting)
      {
        timer=(reference+timer)-curtime;
        if(timer<0) timer=0;  
        counting=0;
        Mix_HaltChannel(0);
      }
      else if(timer != 0)
      {
        Mix_PlayChannel(-1,egg_resources[EGGRES_TICK].data.sound,0);
        counting=1;
      }
      break;

    case SDL_MOUSEUP_H(SDL_BUTTON_WHEELUP,SDL_BUTTON_RMASK):
      Mix_PlayChannel(-1,egg_resources[EGGRES_TICK].data.sound,0);
      timer+=MINUTE_TICKS*10L;
      break;

    case SDL_MOUSEUP_H(SDL_BUTTON_WHEELDOWN,SDL_BUTTON_RMASK):
      Mix_PlayChannel(-1,egg_resources[EGGRES_TICK].data.sound,0);
      timer-=MINUTE_TICKS*10L;
      if(timer<0) timer=0;
      break;

    case SDL_MOUSEUP_H(SDL_BUTTON_WHEELUP,0):
      Mix_PlayChannel(-1,egg_resources[EGGRES_TICK].data.sound,0);
      timer+=MINUTE_TICKS;
      break;

    case SDL_MOUSEUP_H(SDL_BUTTON_WHEELDOWN,0):
      Mix_PlayChannel(-1,egg_resources[EGGRES_TICK].data.sound,0);
      timer-=MINUTE_TICKS;
      if(timer<0) timer=0;
      break;

    case SDL_QUIT:
      running=0;
      continue;

    default:
      break;
    }

    toffset=(reference+timer)-curtime;
    if(toffset<=0)
    {
      if(counting)
          Mix_PlayChannel(-1,egg_resources[EGGRES_ALARM].data.sound,0);

      timer=0;
      reference=curtime;
      toffset=0;
    }

    mod_time(toffset,buf);
    SDL_SetClipRect(screen,&t_area);
    SDL_BlitSurface(egg_resources[EGGRES_BACKGND].data.graphic,
      NULL,screen,NULL);
//    stringRGBA(screen,t_area.x,t_area.y,buf,0,0,0,SDL_ALPHA_OPAQUE);
    stringRGBA(screen,t_area.x,t_area.y,buf,0,0,0);
    SDL_SetClipRect(screen,NULL);
    SDL_UpdateRect(screen,t_area.x,t_area.y,t_area.w,t_area.h);
    SDL_Delay(20);
  }

  SDL_FreeResources(egg_resources);
  Mix_CloseAudio();

  SDL_Quit();
  return(0);
}

void checker_fill(SDL_Surface *s, int x, int y, int size,
  SDL_Color cs1, SDL_Color cs2)
{
  Uint32 c1=SDL_MapRGB(s->format,cs1.r,cs1.g,cs1.b),
         c2=SDL_MapRGB(s->format,cs2.r,cs2.g,cs2.b);

  SDL_Rect r={x,y,size*2,size*2};

  {
    // Draw first checkerbox
    SDL_FillRect(s,&r,c1);
    r.x=x+size;
    r.w=size;
    r.h=size;
    SDL_FillRect(s,&r,c2);
    r.x=x;
    r.y=y+size;
    SDL_FillRect(s,&r,c2);
    r.x=x;
    r.y=y;
    r.w=size*2;
    r.h=size*2;
  }

  while((r.w < s->w) || (r.h < s->h))
  {
    SDL_Rect r2=r;
    if(r.w < s->w)
    {
      // Double checkboxes horizontally
      r2.x+=r2.w;
      SDL_BlitSurface(s,&r,s,&r2);
      r.w*=2;
    }
    else
    {
      // Double checkboxes vertically
      r2.y+=r2.h;
      SDL_BlitSurface(s,&r,s,&r2);
      r.h*=2;
    }
  }
}

void mod_time(Sint32 itime, char *buf)
{
  static const int powers[]={10,100,60,60,-1};
  static const char *labels[]={"",".",":",":",NULL};
  static const char *cmdstrs[]={"%02d","%02d","%02d","%02d",NULL};
//  static const int    powers[] = { SECOND_TICKS,    60,    60,  -1};
//  static const char  *labels[] = {    "",   ":",   ":",NULL};
//  static const char *cmdstrs[] = {"%02d","%02d","%02d",NULL};
  int values[10],n,pos=0;

  for(n=0; powers[n]>=0; n++)
  {
    int imod=-1;
    itime/=powers[n];

    if(powers[n+1]>=0) imod=powers[n+1];

    if(imod>0)
      values[n]=itime%imod;
    else
      values[n]=itime;
  }

  while(n-- > 0)
  {
    pos+=sprintf(buf+pos,cmdstrs[n],values[n]);
    pos+=sprintf(buf+pos,"%s",labels[n]);
  }
}


